<?php if (!defined('PmWiki')) exit();

/* googlesitemap.php

 * Written by (c) 2023 David Cooke
 * 
 * This text is written for PmWiki; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version. See pmwiki.php for full details
 * and lack of warranty.
 * 
 * A recipe to create an xml sitemap for a PmWiki website
 *
 * The script is set to operate correctly whether clean urls are in use or not.
 * A clean sitemap.xml is created whenever ?action=sitemap is selected from any 
 * page and the script will return to the same page.
 
 * The sitemap - sitemap.xml will be placed in the root of the website.
 
 * The script is valid up to PHP8.2 and PmWiki 2.3.26
 *
 * Change notes
 * 2023-10-18 Original issue
 * 2023-10-21 Correction of a couple of stupid errors (wrongly setting return address where clean URL not in use) and addition of changefreq autoset.
 * 2023-10-29 Minor bugfix
 */
 
$RecipeInfo['googlesitemap']['Version'] = '2023-10-29';

global $WikiLibDirs, $EnablePathInfo, $prior, $count, $SitemapMaxcSaves, $SitemapAuto, $EnablePostAttrClearSession;
$EcsTemp = $EnablePostAttrClearSession;
$EnablePostAttrClearSession = 0;
$PageAttributes['prior'] = '$[Set sitemap priority:]';

if (!isset ($SitemapMaxSaves)) $SitemapMaxSaves=20;
if (!isset ($SitemapAuto)) $SitemapAuto = 0;

// Add a custom wikipage storage location.
$WikiLibDirs = array(&$WikiDir,
    new PageStore('$FarmD/cookbook/googlesitemap/wikilib.d/{$FullName}'),
    new PageStore('$FarmD/wikilib.d/{$FullName}'));
// Add edit counter location to count to sitemap update
SDV($AutoSiteMapCount, "$WorkDir/.sitemapcount");
if (!file_exists($AutoSiteMapCount)) {
    touch($AutoSiteMapCount);
}

$ret = ( isset($_SERVER['HTTPS']) && $_SERVER['HTTPS']=='on' ? 'https' : 'http' ) . '://' .  $_SERVER['HTTP_HOST'] . $_SERVER["REQUEST_URI"];
$root = strtok($ret, '?');

if ($EnablePathInfo == 1) {
	# clean urls - remove /$Group/$Name
	$ret =  strtok($ret, '?');
	$txt1 = substr( $root, 0, strrpos( $root, '/', -2 ) )."/";
	$root = substr( $txt1, 0, strrpos( $txt1, '/', -2 ) )."/";
} else {
	# std urls
	$ret =  str_replace("?action=sitemap", "", $ret);
	$root = $root . "?n=";
}


if ( ($action == "edit") and ($SitemapAuto <> 0) ) {
	$smc  = fopen( $AutoSiteMapCount, 'r' );
	$count = fgets( $smc, 10 );
	fclose( $smc );
	$count = abs( intval( $count ) ) + 1;
	$smc = fopen( $AutoSiteMapCount, 'w' );
	fwrite( $smc, $count );
	fclose( $smc );
	$smc  = fopen( $AutoSiteMapCount, 'r' );
	$count = fgets( $smc, 10 );
	fclose( $smc );
	if ($count > $SitemapMaxSaves) {
		$count = 0;
		$smc = fopen( $AutoSiteMapCount, 'w' );
		fwrite( $smc, $count );
		fclose( $smc );
		$ret = $ret . "?action=sitemap";
		header ("Location: ".$ret);
	}
}
		
SDV($HandleActions['sitemap'], 'BuildList');

SDVA($SitemapSearchPatterns, array());
$SitemapSearchPatterns[] = '!\.(All)?Recent(Changes|Uploads|Pages)$!';
$SitemapSearchPatterns[] = '!\.Group(Print)?(Header|Footer|Attributes)$!';
$SitemapSearchPatterns[] = '!^PmWiki\.!';
$SitemapSearchPatterns[] = '!^Site\.!';
$SitemapSearchPatterns[] = '!^SiteAdmin\.!';
$SitemapSearchPatterns[] = '!\.SideBar!';


if (!isset($EnablePathInfo)) $EnablePathInfo = 0;

function BuildList()
{
    global $sitemap, $root, $urlform, $SitemapSearchPatterns, $ret, $EnablePathInfo, $now;
	$sitemap = fopen("../sitemap.xml", "w") or die("Unable to open file!");
    $txt = '<?xml version="1.0" encoding="UTF-8"?>
<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">
';
    fwrite($sitemap, $txt);

    $t = array();
    $t = @ListPages($SitemapSearchPatterns);
    foreach ($t as $i => $pn) {
    $page= ReadPage($pn, READPAGE_CURRENT);
    if (!$page) continue;
	
	if (!isset($page['prior'])) {
		$prior = 0.8;
	} else {
		$prior = $page['prior'];
	}
	if (!$prior) $prior = 0.8;
	$nametext = $page['name'];
	$fullname = str_replace(".", "/", $nametext);
	if ($EnablePathInfo == 0) {
		$url = $root.$page['name'];
	} else {
		$url = $root.$fullname;
	}
    $txt = "<url>
";
    fwrite($sitemap, $txt);
    $txt = "<loc>".$url."</loc>
";
    fwrite($sitemap, $txt);
    $txt = "<lastmod>".PSFT("%Y-%m-%d", $page['time'])."</lastmod>
";
    fwrite($sitemap, $txt);
	$timestamp1 = strtotime("now");
	$timestamp2 = strtotime($page['time']);
	$diff = $timestamp2 - $timestamp1;
	if ($diff > 60 * 60 * 24 * 7 * 52) {
		$freq = "yearly";
	} elseif ($diff > 60 * 60 * 24 * 7) {
		$freq = "monthly";
	} elseif ($diff > 60 * 60* 24) {
		$freq = "weekly";
	} else {
		$freq="daily";
	}
    $txt = "<changefreq>".$freq."</changefreq>
";
    fwrite($sitemap, $txt);
    $txt = "<priority>".$prior."</priority>
";
    fwrite($sitemap, $txt);
    $txt = "</url>
";
    fwrite($sitemap, $txt);
}
	$txt = "</urlset>
";
    fwrite($sitemap, $txt);
    fclose($sitemap);
	$EnablePostAttrClearSession = $EcsTemp;
	header ("Location: ".$ret);
	}